#!/usr/bin/env python

# Copyright (C) 2008  Peter Gill - peter@majorsilence.com
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import time
import subprocess
import select
import signal

import gobject

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade

import iso_execute
import iso_helper

print "Version 1.3"
# tested with:
# uif2iso 0.1.3 
# daa2iso 0.1.4
# nrg2iso v0.4


#TODO: Add support for b5i2iso - BlindWrite bin files (.b5i or .bwi)
#TODO: Add support for ccd2iso - Clone CD disc Image (.img)
#TODO: Add support for cdi2iso - DiscJuggler Image (.)
#TODO: Add support for mdf2iso - (.mdf)
#TODO: Add support for pdi2iso - Instant Copy Bin (.)
#TODO: Add bin2iso support - (.bin)


class Uif2Iso(object):
    def __init__(self):
        self.gladefile = gtk.glade.XML("gui.glade")
        self.gladefile.signal_autoconnect(self)

        # windows and dialogs
        self.main_window = self.gladefile.get_widget("mainwindow")
        self.about_dialog = self.gladefile.get_widget("aboutdialog1")
        self.progress = self.gladefile.get_widget("progress")
        self.password_dialog = self.gladefile.get_widget("password_dialog")
        
        # drag and drop
        self.main_window.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT |
                    gtk.DEST_DEFAULT_DROP, [("text/plain", 0, 80)], gtk.gdk.ACTION_COPY)
        self.main_window.connect('drag_data_received', self.drag_and_drop)
        
        self.statusbar = self.gladefile.get_widget("statusbar1")
        self.statusbar_context_id = self.statusbar.get_context_id("display info")
        
        self.menu_bar = self.gladefile.get_widget("menubar1")
        self.progress_bar = self.gladefile.get_widget("progressbar1")
        self.cancel_button = self.gladefile.get_widget("cancel")  
        self.add_button = self.gladefile.get_widget("add")
        self.remove_button = self.gladefile.get_widget("remove")
        self.convert_button = self.gladefile.get_widget("convert")
        
        # Treeview Stuff
        self.liststore = gtk.ListStore(str)
        #self.treeview = gtk.TreeView(self.liststore)
        self.treeview = self.gladefile.get_widget("treeview1")
        self.treeview.set_model(self.liststore)
        
        # Add cell and column.  Needed to view
        # data added to treeview.
        self.cell = gtk.CellRendererText()
        self.tvcolumn = gtk.TreeViewColumn("Files", self.cell, text=0)
        self.treeview.append_column(self.tvcolumn)
        
        #Password Dialog
        self.password_input= self.gladefile.get_widget("password_input")
        
        self.count=0
        self.percent=0
        self.program_ended=False
        self.cancel = False

    # Helper functions here        
    def update_gui(self):
        # Should be doing this another way
        while gtk.events_pending():
            gtk.main_iteration()
             
    def refresh(self, handle=None, file_info=None):
        # file_info is a tuple (int, int, int, str) Eg. "current file(3)" of "all files(6)" - "size (500 mb)", "file type(daa)"
        if handle == None:
            self.program_ended=True
            return
        return_value=1 # 0: nothing to read, 1: program ended
        percent=0
        file_size = file_info[2]
        file_type = file_info[3]
        if file_type=="uif" or file_type=="daa":
            uif_daa_refresh = iso_execute.uif_daa_refresh(self.password_dialog, self.password_input, self.progress_bar, file_info, handle)
        
        while handle.poll()==None:
            sout, sin, serr = select.select([handle.stdout], [], [handle.stderr], 0)
            #print "length: ", len(sout)
            if self.cancel:
                iso_execute.cancel_prog(handle)
                
            if file_type=="uif" or file_type=="daa":
                uif_daa_refresh.process(sout, sin, serr)
            else:
                self.progress_bar.pulse()
                time.sleep(0.1)
            #print "percent-%s: -count: %s" % (file_info[0], self.percent)
            self.update_gui()
            if file_size > 200:        
                time.sleep(0.01) # keep python from killing the cpu
                
        self.program_ended = True
        self.percent=0
        self.progress_bar.set_fraction(0.0) # reset to 0 before next file starts
        print "program ended - refresh: ", self.program_ended
        return 
    
    
    def set_conversion_mode(self, x=None):
        """
        Takes value x as True or False
        """
        if x is None:
            return
        self.menu_bar.set_sensitive(x)
        self.add_button.set_sensitive(x)
        self.remove_button.set_sensitive(x)
        self.convert_button.set_sensitive(x)
        
    # GUI Callbacks Start Here     
    def on_mainwindow_delete_event(self, w=None, e=None):
        gtk.main_quit()
    
    def on_cancel_clicked(self, w=None):
        self.cancel=True
    
    def on_add_clicked(self, w=None):
        filename=None
        dialog = gtk.FileChooserDialog(title="Add uif file",
                action=gtk.FILE_CHOOSER_ACTION_OPEN,
                buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        
        filter = gtk.FileFilter()
        filter.set_name("All supported files")
        filter.add_pattern("*.uif")
        filter.add_pattern("*.daa")
        filter.add_pattern("*.nrg")
        dialog.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name("uif files")
        filter.add_pattern("*.uif")
        dialog.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name("daa files")
        filter.add_pattern("*.daa")
        dialog.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name("nrg files")
        filter.add_pattern("*.nrg")
        dialog.add_filter(filter)
        
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)
        
        response=dialog.run()
        if response==gtk.RESPONSE_OK:
            filename=dialog.get_filename()
            print filename
        dialog.destroy()
        if filename is None:
            return
                
        model = self.treeview.get_model()
        model.append([filename])
        #print self.liststore[(0,)][0], ": 1"
        #for count, row in enumerate(self.liststore):
        #    print row[0], ": for loop", count
        print "added", filename
    
    def on_remove_clicked(self, w=None):
        print "remove"
        selection = self.treeview.get_selection()
        model, iter = selection.get_selected()
        if iter:
            model.remove(iter)
        return
    
    def on_convert_clicked(self, w=None):
        print "convert"
        file_list=[]
        for x in self.liststore: 
            file_list.append(x[0]) # first column
        if len(file_list) is 0:
             return
            
        # Run through all files in list
        # and convert them to iso.
        # Called from on_convert_clicked
        self.statusbar.pop(self.statusbar_context_id)
        
        for file_name in file_list:
            if (iso_execute.check_exists(file_name) != 0): # 0 is overwrite or good to go
                file_list.remove(file_name)
        
        # Deactivate menubar and buttons while converting
        self.set_conversion_mode(False)
                
        self.progress_bar.show()
        self.cancel_button.show()
        list_length = len(file_list)

        for count, file_name in enumerate(file_list):
            if self.cancel:
                break
            count=count+1 # start the count at 1 to meet length in refresh
            size = os.stat(file_name).st_size/1024000
            file_type=iso_execute.get_file_type(file_name)
            
            handle = iso_execute.execute(file_name, file_type)
            #timer=gobject.timeout_add(1000, self.refresh, handle,(count, list_length, size, file_type))
            self.refresh(handle, (count, list_length, size, file_type))
        
        self.progress_bar.hide()
        self.cancel_button.hide()
        
        #Reactivate menubar and buttons after convert
        self.set_conversion_mode(True)
        
        if self.cancel:
            msg = "Conversion cancelled."
        else:
            msg = "Conversion finished."
        iso_helper.info_message(msg)
        
        self.cancel=False # so we can try to convert again
        self.statusbar.push(self.statusbar_context_id, "Finished...")
        return False   
        
    def drag_and_drop(self, w, drag_context, x, y, selection, info, time):
        print "drag_and_drop"
        ext_list = ["daa", "uif", "nrg"]
        model = self.treeview.get_model()
        data = selection.data.split("\n")
        for x in data:
            add = x.strip()
            add = add[7:] # remove file://
            
            # drag and drop adds %20 for blank spaces, which does not work with uif2iso and daa2iso
            add = add.replace("%20", " ") 
            ext = iso_execute.get_file_type(add)
            if ext in ext_list:
                model.append([add])
    
    def on_about_activate(self, w=None):
        self.about_dialog.run()
        self.about_dialog.destroy()
    
if __name__ == "__main__":
    gui = Uif2Iso()
    gtk.main()
