#!/bin/env python

# Copyright (C) 2008  Peter Gill - peter@majorsilence.com
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import time
import subprocess
import select
import signal

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade

import iso_helper

def cancel_prog(handle=None):
    if handle is not None:
        os.kill(handle.pid,signal.SIGKILL)
    return

def get_file_type(file_name):
    return os.path.splitext(file_name)[1][1:]
    
def check_exists(file_name=None):
    if file_name is None:
        return False
        
    return_value = 0 # 0 means no iso file with the same name present, or if present overwrite
    iso_file = os.path.splitext(file_name)[0] + ".iso"
    print iso_file
    if os.path.isfile(iso_file):
        msg = "File \"%s\" alreay exists.  Do you wish to overwrite." % os.path.basename(iso_file)
        dialog=gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, msg)
        response = dialog.run()
        print response
        if response==gtk.RESPONSE_YES:
            os.remove(iso_file)
        else:
            return_value=1
        dialog.destroy()
    return return_value

def execute(file_name=None, file_type=None):
    out_file = os.path.splitext(file_name)[0] + ".iso"
    print out_file
    app=None
    
    if (file_type == "daa"):
        app="daa2iso"
    elif (file_type == "uif"):
        app="uif2iso"
    elif (file_type == "nrg"):
        app="nrg2iso"
    
    path_list=os.environ["PATH"].split(":")
    path_list.append(os.getcwd())
    for x in path_list:
        launch = os.path.join(x, app)
        print app
        try:
            handle = subprocess.Popen([launch, file_name, out_file], shell=False, stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
        except:
            print "Error launching program.  Program not found in path."
        else:
            print "Program launched:", app, file_name, out_file
            return handle
    return None # No program running

class uif_daa_refresh:
    def __init__(self, password_dialog, password_input, progress_bar, file_info, handle):
        
        self.return_value=1 # 0: nothing to read, 1: program ended
        self.output=""
        self.readline=True
        self.password_check=False
        self.percent=0
        self.password_dialog = password_dialog
        self.password_input=password_input
        self.progress_bar=progress_bar
        self.file_info = file_info
        self.file_type = file_info[3]
        self.handle= handle

    def process(self, sout=None, stdin=None, stderr=None):
        read=""
        percent = 0
        for line in sout:
            if self.readline == True:
                if self.password_check==True:
                    self.password_check=False # only check for password once
                    read = line.read(54)
                    if read == "- the input file is protected by password, insert it: ":
                        print "do password"
                        
                        password_text=""
                        response=self.password_dialog.run()
                        if response==2: # 1-Cancel, 2-OK
                            password_text=self.password_input.get_text()
                            self.password_input.set_text("")
                            print "password text: ", password_text
                        self.password_dialog.hide()
                        self.handle.stdin.write(password_text+"\n")
                        self.handle.stdin.flush()
                else:
                    read = line.readline()
                
                # Set whether to test next line to see if the file is password
                # protected based on file type
                if self.file_type == "daa":
                    if read.startswith("- create"):
                        self.password_check=True
                elif self.file_type == "uif":
                    if read.startswith("  hash"):
                        print "hash line found.  Next run through check to see if there is a password."
                        self.password_check=True
                #TODO: check for: "- the output file already exists, do you want to overwrite it (y/N)? "
                # needs to be checked in case two files have the same name except file extension
                 
                if read.find("start unpacking:") is not -1: # -1 if not found, any other number and an error is found
                    self.readline = False
                elif read.startswith("Error:"):
                    iso_helper.warn_message(read)
                elif read.find("Alert:") is not -1:
                    iso_helper.warn_message(read)       
            else:
                read = line.read(7) # 2 white spaces, 3 numbers, 1 percent sign, 1 unknown(?)/white space?
                    
                #TODO: Test to make sure these errors are handles properly
                if read.startswith("Error:"):
                    read = read + line.readline()
                    warn_message(read)
                    break
                
                read = read.replace("%", "").strip()
                if read.isdigit():
                    percent = int(read)
                if percent >= self.percent:
                    self.percent = percent
                    text = str(self.percent) + "%"
                    self.progress_bar.set_fraction(self.percent/100.0)
                    self.progress_bar.set_text("%s of %s - %s" % (self.file_info[0], self.file_info[1], text))
                if self.percent == "100":
                    self.readline=True
            print read
            